const request = require('request');
const { Router } = require('express');

module.exports =
  new Router()
    .get('/', (req, res) => {

      const { flowId, apiKey, baseUrl } = req.query;
      if (!flowId) return res.status(422).send({ error:'missing required field: flowId' });
      if (!apiKey) return res.status(422).send({error: 'missing required field: apiKey'});

      const auth = `Basic ${Buffer.from('X:' + apiKey).toString('base64')}`;
      const options = {
        url: `${baseUrl}/flows/${flowId}`,
        method: 'GET',
        headers: {
          Accept: 'application/json',
          Authorization: auth,
        }
      };
      request(options, (err, response, body) => {
        if (err) {
          return res.status(422).send({ error: err.message });
        }
        else if (response.statusCode !== 200) {
          let message = '';
          try {
            message = JSON.parse(body).error;
          }
          catch (e) {
            message = `Error (${response.statusCode}); unable to parse response from LeadConduit`;
          }
          return res.status(response.statusCode).send({ error: message });
        }
        res.status(200).send(body);
      });
    })
    .get('/meta/', (req, res) => {
      const { flowId, sourceId, baseUrl } = req.query;

      if (!flowId || !sourceId) {
        let errorMessage = 'missing required field:';
        if (!flowId) errorMessage += ' flowId';
        if (!sourceId) errorMessage += ' sourceId';
        return res.status(422).send({ error: errorMessage });
      }

      const options = {
        url: `${baseUrl}/flows/${flowId}/sources/${sourceId}/meta`,
        method: 'GET',
        headers: {
          Accept: 'application/json',
        }
      };
       request(options, (err, response, body) => {
        if (err) {
          return res.status(422).send({ error: err.message });
        }
        else if (response.statusCode !== 200) {
          let message = '';
          try {
            return res.status(200).send({ error: JSON.parse(body).error });
          }
          catch (e) {
            return res.status(response.statusCode).send({ error: `Error (${response.statusCode}); unable to parse response from LeadConduit`});
          }
        }
        res.status(200).send(body);
      });
    });
